@@ -160,6 +160,44 @@ struct RF_ScorerWrapper {
     }
 };
 
+struct RF_UncachedScorerWrapper {
+    RF_UncachedScorerFunc scorer_func;
+
+    explicit RF_UncachedScorerWrapper(RF_UncachedScorerFunc scorer_func_) : scorer_func(scorer_func_)
+    {}
+
+    RF_UncachedScorerWrapper(const RF_UncachedScorerWrapper&) = delete;
+    RF_UncachedScorerWrapper& operator=(const RF_UncachedScorerWrapper&) = delete;
+
+    RF_UncachedScorerWrapper(RF_UncachedScorerWrapper&& other) : scorer_func(other.scorer_func)
+    {}
+
+    RF_UncachedScorerWrapper& operator=(RF_UncachedScorerWrapper&& other)
+    {
+        if (&other != this) scorer_func = other.scorer_func;
+
+        return *this;
+    };
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, double score_cutoff,
+              double score_hint, double* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.f64(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, int64_t score_cutoff,
+              int64_t score_hint, int64_t* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.i64(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, size_t score_cutoff,
+              size_t score_hint, size_t* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.sizet(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
+};
+
 template <typename T>
 bool is_lowest_score_worst(const RF_ScorerFlags* scorer_flags)
 {
@@ -532,7 +570,7 @@ static Matrix cdist_two_lists_impl(const RF_ScorerFlags* scorer_flags, const RF_
 
         for (auto it = none_begin; it != row_idx.end(); it++)
             for (int64_t col = 0; col < cols; ++col)
-                matrix.set(*it, col, worst_score);
+                matrix.set(*it, col, worst_score * score_multiplier);
 
         row_idx.erase(none_begin, row_idx.end());
 
@@ -619,6 +657,14 @@ static Matrix cdist_two_lists_impl(const RF_ScorerFlags* scorer_flags, const RF_
     else {
         run_parallel(workers, rows, 1, [&](int64_t row, int64_t row_end) {
             for (; row < row_end; ++row) {
+                if(queries[row].is_none())
+                {
+                    for (int64_t col = 0; col < cols; ++col)
+                        matrix.set(row, col, worst_score * score_multiplier);
+
+                    continue;
+                }
+
                 RF_ScorerFunc scorer_func;
                 PyErr2RuntimeExn(scorer->scorer_func_init(&scorer_func, kwargs, 1, &queries[row].string));
                 RF_ScorerWrapper ScorerFunc(scorer_func);
@@ -638,3 +684,35 @@ static Matrix cdist_two_lists_impl(const RF_ScorerFlags* scorer_flags, const RF_
 
     return matrix;
 }
+
+template <typename T>
+static Matrix cpdist_cpp_impl(const RF_Kwargs* kwargs, RF_Scorer* scorer,
+                              const std::vector<RF_StringWrapper>& queries,
+                              const std::vector<RF_StringWrapper>& choices, MatrixType dtype, int workers,
+                              T score_cutoff, T score_hint, T score_multiplier, T worst_score)
+{
+    int64_t rows = queries.size();
+    int64_t cols = 1;
+    Matrix matrix(dtype, static_cast<size_t>(rows), static_cast<size_t>(cols));
+
+    if (queries.empty() || choices.empty()) return matrix;
+
+    // TODO: Adjust the batch size based on the query, choices & hardware available
+    const int batchSize = 16;
+
+    run_parallel(workers, rows, batchSize, [&](int64_t row, int64_t row_end) {
+        for (; row < row_end; ++row) {
+            T score;
+            if (choices[row].is_none() || queries[row].is_none())
+                score = worst_score;
+            else
+                RF_UncachedScorerWrapper(scorer->uncached_scorer_func)
+                    .call(&queries[row].string, &choices[row].string, kwargs, score_cutoff, score_hint,
+                          &score);
+
+            matrix.set(row, 0, score * score_multiplier);
+        }
+    });
+
+    return matrix;
+}
